// ------------------------------------------------------------------------
// Copyright 2021 The Dapr Authors
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//     http://www.apache.org/licenses/LICENSE-2.0
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
// ------------------------------------------------------------------------

namespace Dapr.Actors
{
    using System;

    /// <summary>
    /// Exception for Remote Actor Method Invocation.
    /// </summary>
    [Serializable]
    public class ActorMethodInvocationException : DaprApiException
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="ActorMethodInvocationException"/> class.
        /// </summary>
        public ActorMethodInvocationException()
            : base(Constants.ErrorActorInvokeMethod, false)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="ActorMethodInvocationException"/> class.
        /// </summary>
        /// <param name="message">The error message that explains the reason for the exception.</param>
        /// <param name="isTransient">True, if the exception is to be treated as an transient exception.</param>
        public ActorMethodInvocationException(string message, bool isTransient)
            : base(message, Constants.ErrorActorInvokeMethod, isTransient)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="ActorMethodInvocationException"/> class with a specified error
        /// message and a reference to the inner exception that is the cause of this exception.
        /// </summary>
        /// <param name="message">The error message that explains the reason for the exception.</param>
        /// <param name="innerException">The exception that is the cause of the current exception.</param>
        /// <param name="isTransient">True, if the exception is to be treated as an transient exception.</param>
        public ActorMethodInvocationException(string message, Exception innerException, bool isTransient)
            : base(message, innerException, Constants.ErrorActorInvokeMethod, isTransient)
        {
        }
    }
}
